% Main script for Object Recoloring - v4.0
function imRecolored = ObjectRecoloring(imName,maskName,type,color)
    % Check input
    if ((nargin<2) || (nargin>4))
        error('Usage: imRecolored = ObjectRecoloring(imName, maskName [,options])');
    end
    if (nargin<3)
        type='auto';
    end
    if (strcmp(type,'color'))
        if ((nargin~=4) || (isempty(color)))
            error('Usage: imRecolored = ObjectRecoloring(imName, maskName [,options])');
        end
        if ((size(color,1)~=1) || (size(color,2)~=3))
            error('Error, color should be a 1-by-3 matrix: [R G B]');
        end
    end

    % Define options
    %
    %    * RBREscale, RBREmaxItr, RBREthr: scale, maximum number of
    %   iterations and threshold for the Robust Body Reflectance Estimation
    %
    %    * errThr, delta, maxItr: threshold, delta and maximum number of
    %    iterations for the MIDR Estimation
    %
    options = struct('RBREscale',5, 'RBREmaxItr',10, 'RBREthr',300, 'errThr',10, 'delta',3, 'maxItr',10);
    
    % Read image and mask
    im = imread(imName);
    im = double(im);
    mask = imread(maskName);
    mask = logical(mask);

    % Gamma correction
    gamma = 2.4;
    imLinear = gammaCorrection(im,gamma);

    % MIDR Estimation
    [MbD, MsD, MbS, MsS, Cb, LD, LS] = MIDR_Estimation(imLinear,mask,options);

    % Show Results - change Cb to change color (RGB value)
    if(strcmp(type,'color'))
        imRecolored = editImage(imLinear,mask,MbD, MsD, MbS, MsS, gammaCorrection(color,gamma), LD, LS);
    else
        imRecolored = editImage(imLinear,mask,MbD, MsD, MbS, MsS, Cb, LD, LS);
    end
    imRecolored = (uint8(real(gammaCorrection(imRecolored,1/gamma))));
end
