% Object Recoloring DEMO - v3.0
function ObjectRecoloringDEMO()

    % Define options
    %
    %    * RBREscale, RBREmaxItr, RBREthr: scale, maximum number of
    %   iterations and threshold for the Robust Body Reflectance Estimation
    %
    %    * errThr, delta, maxItr: threshold, delta and maximum number of
    %    iterations for the MIDR Estimation
    %
    options = struct('RBREscale',5, 'RBREmaxItr',10, 'RBREthr',300, 'errThr',10, 'delta',3, 'maxItr',10);

    % Read images and masks
    im1 = imread('Images/Italy.jpg');
    im2 = imread('Images/Saturn.jpg');
    im3 = imread('Images/Cups.jpg');
    im4 = imread('Images/Airplane.jpg');
    im1 = double(im1);
    im2 = double(im2);
    im3 = double(im3);
    im4 = double(im4);
    mask1 = imread('Images/Italy.bmp');
    mask2 = imread('Images/Saturn.bmp');
    mask3 = imread('Images/Cups.bmp');
    mask4 = imread('Images/Airplane.bmp');
    mask1 = logical(mask1);
    mask2 = logical(mask2);
    mask3 = logical(mask3);
    mask4 = logical(mask4);

    % Gamma correction
    gamma = 2.4;
    imLinear1 = gammaCorrection(im1,gamma);
    imLinear2 = gammaCorrection(im2,gamma);
    imLinear3 = gammaCorrection(im3,gamma);
    imLinear4 = gammaCorrection(im4,gamma);
    
    im1 = (uint8(real(im1)));
    im2 = (uint8(real(im2)));
    im3 = (uint8(real(im3)));
    im4 = (uint8(real(im4)));
    
    % DRM_Estimation
    disp('--- DRM Estimation ---');
    tic;
    [Mb1, Ms1, CC1, LD1] = DRM_Estimation(imLinear1,mask1,options);
    toc;
    imRecolored1 = editImage(imLinear1, mask1, Mb1, Ms1, zeros(size(Mb1)),zeros(size(Ms1)),[0 0 255], LD1,[1 1 1]); %blue
    imRecolored1 = (uint8(real(gammaCorrection(imRecolored1,1/gamma))));
    imRecolored2 = editImage(imLinear1, mask1, Mb1, Ms1, zeros(size(Mb1)),zeros(size(Ms1)),[0 255 0], LD1,[1 1 1]); %green
    imRecolored2 = (uint8(real(gammaCorrection(imRecolored2,1/gamma))));
    imRecolored3 = editImage(imLinear1, mask1, Mb1, Ms1, zeros(size(Mb1)),zeros(size(Ms1)),[255 255 0], LD1,[1 1 1]); %yellow
    imRecolored3 = (uint8(real(gammaCorrection(imRecolored3,1/gamma))));
    imRecolored4 = editImage(imLinear1, mask1, Mb1, Ms1, zeros(size(Mb1)),zeros(size(Ms1)),CC1, LD1,[1 1 1]); %original
    imRecolored4 = (uint8(real(gammaCorrection(imRecolored4,1/gamma))));
    imRecolored5 = editImage(imLinear1, mask1, Mb1, Ms1, zeros(size(Mb1)),zeros(size(Ms1)),[255 69 0], LD1,[1 1 1]); %orange
    imRecolored5 = (uint8(real(gammaCorrection(imRecolored5,1/gamma))));
    imRecolored6 = editImage(imLinear1, mask1, Mb1, Ms1, zeros(size(Mb1)),zeros(size(Ms1)),[128 0 128], LD1,[1 1 1]); %purple
    imRecolored6 = (uint8(real(gammaCorrection(imRecolored6,1/gamma))));
    imRecolored7 = editImage(imLinear1, mask1, Mb1, Ms1, zeros(size(Mb1)),zeros(size(Ms1)),[255 0 0], LD1,[1 1 1]); %red
    imRecolored7 = (uint8(real(gammaCorrection(imRecolored7,1/gamma))));

    % Show DRM Estimation
    figure(1);
    set(gcf,'numbertitle','off','name','DRM Estimation')
    subplot(2,4,1), imshow(im1)
    title('Original','FontSize',14,'FontWeight','bold')
    subplot(2,4,2), imshow(imRecolored1)
    subplot(2,4,3), imshow(imRecolored2)
    subplot(2,4,4), imshow(imRecolored3)
    subplot(2,4,5), imshow(imRecolored4)
    title('Reconstruction','FontSize',14,'FontWeight','bold')
    subplot(2,4,6), imshow(imRecolored5)
    subplot(2,4,7), imshow(imRecolored6)
    subplot(2,4,8), imshow(imRecolored7)
    
    
    % MIDR Estimation
    disp('--- MIDR Estimation ---');
    tic;
    [MbD2, MsD2, MbS2, MsS2, CC2, LD2, LS2] = MIDR_Estimation(imLinear2,mask2,options);
    toc;
    imRecolored1 = editImage(imLinear2, mask2, MbD2, MsD2, MbS2, MsS2, [0 0 255], LD2, LS2); %blue
    imRecolored1 = (uint8(real(gammaCorrection(imRecolored1,1/gamma))));
    imRecolored2 = editImage(imLinear2, mask2, MbD2, MsD2, MbS2, MsS2, [0 255 0], LD2, LS2); %green
    imRecolored2 = (uint8(real(gammaCorrection(imRecolored2,1/gamma))));
    imRecolored3 = editImage(imLinear2, mask2, MbD2, MsD2, MbS2, MsS2, [255 255 0], LD2, LS2); %yellow
    imRecolored3 = (uint8(real(gammaCorrection(imRecolored3,1/gamma))));
    imRecolored4 = editImage(imLinear2, mask2, MbD2, MsD2, MbS2, MsS2, CC2, LD2, LS2); %original
    imRecolored4 = (uint8(real(gammaCorrection(imRecolored4,1/gamma))));
    imRecolored5 = editImage(imLinear2, mask2, MbD2, MsD2, MbS2, MsS2, [255 69 0], LD2, LS2); %orange
    imRecolored5 = (uint8(real(gammaCorrection(imRecolored5,1/gamma))));
    imRecolored6 = editImage(imLinear2, mask2, MbD2, MsD2, MbS2, MsS2, [128 0 128], LD2, LS2); %purple
    imRecolored6 = (uint8(real(gammaCorrection(imRecolored6,1/gamma))));
    imRecolored7 = editImage(imLinear2, mask2, MbD2, MsD2, MbS2, MsS2, [255 0 0], LD2, LS2); %red
    imRecolored7 = (uint8(real(gammaCorrection(imRecolored7,1/gamma))));

    % Show MIDR Estimation
    figure(2);
    set(gcf,'numbertitle','off','name','MIDR Estimation')
    subplot(2,4,1), imshow(im2)
    title('Original','FontSize',14,'FontWeight','bold')
    subplot(2,4,2), imshow(imRecolored1)
    subplot(2,4,3), imshow(imRecolored2)
    subplot(2,4,4), imshow(imRecolored3)
    subplot(2,4,5), imshow(imRecolored4)
    title('Reconstruction','FontSize',14,'FontWeight','bold')
    subplot(2,4,6), imshow(imRecolored5)
    subplot(2,4,7), imshow(imRecolored6)
    subplot(2,4,8), imshow(imRecolored7)
    
    % Transfer
    disp('--- Color Transfer ---');
    tic;
    [MbD3, MsD3, MbS3, MsS3, CC3, LD3, LS3] = MIDR_Estimation(imLinear3,mask3,options);
    toc;
    tic;
    [MbD4, MsD4, MbS4, MsS4, CC4, LD4, LS4] = MIDR_Estimation(imLinear4,mask4,options);
    toc;
    imTransf32 = editImage(imLinear2, mask2, MbD2, MsD2, MbS2, MsS2, CC3, LD2, LS2);
    imTransf32 = (uint8(real(gammaCorrection(imTransf32,1/gamma))));
    imTransf43 = editImage(imLinear3, mask3, MbD3, MsD3, MbS3, MsS3, CC4, LD3, LS3);
    imTransf43 = (uint8(real(gammaCorrection(imTransf43,1/gamma))));
    imTransf24 = editImage(imLinear4, mask4, MbD4, MsD4, MbS4, MsS4, CC2, LD4, LS4);
    imTransf24 = (uint8(real(gammaCorrection(imTransf24,1/gamma))));
    imTransf42 = editImage(imLinear2, mask2, MbD2, MsD2, MbS2, MsS2, CC4, LD2, LS2);
    imTransf42 = (uint8(real(gammaCorrection(imTransf42,1/gamma))));
    imTransf23 = editImage(imLinear3, mask3, MbD3, MsD3, MbS3, MsS3, CC2, LD3, LS3);
    imTransf23 = (uint8(real(gammaCorrection(imTransf23,1/gamma))));
    imTransf34 = editImage(imLinear4, mask4, MbD4, MsD4, MbS4, MsS4, CC3, LD4, LS4);
    imTransf34 = (uint8(real(gammaCorrection(imTransf34,1/gamma))));    

    % Show Transfer
    figure(3);
    set(gcf,'numbertitle','off','name','MIDR Color Transfer')
    subplot(3,3,1), imshow(im2)
    title('Original','FontSize',14,'FontWeight','bold')
    subplot(3,3,2), imshow(im3)
    title('Original','FontSize',14,'FontWeight','bold')
    subplot(3,3,3), imshow(im4)
    title('Original','FontSize',14,'FontWeight','bold')
    subplot(3,3,4), imshow(imTransf32)
    subplot(3,3,5), imshow(imTransf43)
    subplot(3,3,6), imshow(imTransf24)
    subplot(3,3,7), imshow(imTransf42)
    subplot(3,3,8), imshow(imTransf23)
    subplot(3,3,9), imshow(imTransf34)
    
end
