% Robust Body Reflectance Estimation
function [Cb PerpCb]=RBRE(f,mask,options)

    indx=find(mask);
    r=f(:,:,1);
    g=f(:,:,2);
    b=f(:,:,3); 
    f2=[r(indx) g(indx) b(indx)];

    [cbSVD PerpCb]=robust(f2,options.RBREscale,options.RBREmaxItr,options.RBREthr);

    Cb=(cbSVD ./ norm(cbSVD)) *255;
    PerpCb=(PerpCb./ norm(PerpCb)).*255;

end

function [cb perp]=robust(f,scale,maxItr,thr)

    % Only consider pixels which have lower intensity than thr
    m = sum(f,2)<thr;
    f2 = f.*repmat(m,[1 3]);
    [~,~,V] = svd(f2'*f2);

    est=[10;10;10];
    itr=0;
    while ((norm(est-V(:,1))>0.0001) && (itr<maxItr))
        est     = V(:,1);
        dist    = sum((f-(f*est)*est').^2,2);
        weight  = exp(-dist./(2*scale^2));    
        wf      = repmat(weight,1,3).*f;
        [~,~,V] = svd(wf'*wf);
        itr=itr+1;
    end

    cb=-(V(:,1)./norm(V(:,1)));

    f_cs=f-(f*cb)*cb';
    [~,~,V]=svd(f_cs'*f_cs);

    perp=-(V(:,1)./norm(V(:,1)));

end
